<?php
/**
 * @package     BreezingForms
 * @author      Markus Bopp
 * @link        http://www.crosstec.de
 * @license     GNU/GPL
*/

// no direct access
defined( '_JEXEC' ) or die( 'Restricted access' );

jimport('joomla.filesystem.file');

require __DIR__ . '/Exception.php';

class telegramBot
{
  const BASE_URL = 'https://api.telegram.org/bot';

  protected $token;

  public function __construct($token)
  {
    $this->token = $token;
    if (is_null($this->token))
      throw new TelegramException('Required "token" key not supplied');

    $this->baseURL = self::BASE_URL . $this->token . DIRECTORY_SEPARATOR;
  }

  /**
   * A simple method for testing your bot's auth token.
   * Returns basic information about the bot in form of a User object.
   *
   * @link https://core.telegram.org/bots/api#getme
   *
   * @return Array
   */
  public function getMe()
  {
    return $this->sendRequest('getMe', array());
  }


  /**
   * Use this method to receive incoming updates using long polling.
   *
   * @link https://core.telegram.org/bots/api#getupdates
   *
   * @param int $offset
   * @param int $limit
   * @param int $timeout
   *
   * @return Array
   */
  public function pollUpdates($offset = null, $timeout = null, $limit = null)
  {
    $params = compact('offset', 'limit', 'timeout');

    return $this->sendRequest('getUpdates', $params);
  }

  /**
   * Send text messages.
   *
   * @link https://core.telegram.org/bots/api#sendmessage
   *
   * @param int            $chat_id
   * @param string         $text
   * @param bool           $disable_web_page_preview
   * @param int            $reply_to_message_id
   * @param KeyboardMarkup $reply_markup
   *
   * @return Array
   */
  public function sendMessage($chat_id, $text, $disable_web_page_preview = false, $reply_to_message_id = null, $reply_markup = null, $parse_mode = null)
  {
    $params = compact('chat_id', 'text', 'disable_web_page_preview', 'reply_to_message_id', 'reply_markup', 'parse_mode');

    return $this->sendRequest('sendMessage', $params);
  }

  /**
   * Forward messages of any kind.
   *
   * @link https://core.telegram.org/bots/api#forwardmessage
   *
   * @param int $chat_id
   * @param int $from_chat_id
   * @param int $message_id
   *
   * @return Array
   */
  public function forwardMessage($chat_id, $from_chat_id, $from_chat_id2)
  {
    $params = compact('chat_id', 'from_chat_id', 'from_chat_id2');

    return $this->sendRequest('forwardMessage', $params);
  }

  /**
   * Send Photos.
   *
   * @link https://core.telegram.org/bots/api#sendphoto
   *
   * @param int            $chat_id
   * @param string         $photo
   * @param string         $caption
   * @param int            $reply_to_message_id
   * @param KeyboardMarkup $reply_markup
   *
   * @return Array
   */
  public function sendPhoto($chat_id, $photo, $caption = null, $reply_to_message_id = null, $replyMarkup = null)
  {
    $data = compact('chat_id', 'photo', 'caption', 'reply_to_message_id', 'reply_markup');

  
    return $this->uploadFile('sendPhoto', $data, $photo);
  }

  /**
   * Send Audio.
   *
   * @link https://core.telegram.org/bots/api#sendaudio
   *
   * @param int            $chat_id
   * @param string         $audio
   * @param int            $reply_to_message_id
   * @param KeyboardMarkup $reply_markup
   *
   * @return Array
   */
  public function sendAudio($chat_id, $audio, $reply_to_message_id = null, $replyMarkup = null)
  {
    $data = compact('chat_id', 'audio', 'reply_to_message_id', 'reply_markup');

    return $this->uploadFile('sendAudio', $data, $audio);
  }

  /**
   * Send Document.
   *
   * @link https://core.telegram.org/bots/api#senddocument
   *
   * @param int            $chat_id
   * @param string         $document
   * @param int            $reply_to_message_id
   * @param KeyboardMarkup $reply_markup
   *
   * @return Array
   */
  public function sendDocument($chat_id, $document, $reply_to_message_id = null, $replyMarkup = null)
  {
    $data = compact('chat_id', 'document', 'reply_to_message_id', 'reply_markup');

    return $this->uploadFile('sendDocument', $data, $document);
  }

  /**
   * Send Sticker.
   *
   * @link https://core.telegram.org/bots/api#sendsticker
   *
   * @param int            $chat_id
   * @param string         $sticker
   * @param int            $reply_to_message_id
   * @param KeyboardMarkup $reply_markup
   *
   * @return Array
   */
  public function sendSticker($chat_id, $sticker, $reply_to_message_id = null, $replyMarkup = null)
  {
    $data = compact('chat_id', 'sticker', 'reply_to_message_id', 'reply_markup');

    
    return $this->uploadFile('sendSticker', $data, $sticker);
  }

  /**
   * Send Video.
   *
   * @link https://core.telegram.org/bots/api#sendvideo
   *
   * @param int            $chat_id
   * @param string         $video
   * @param int            $reply_to_message_id
   * @param KeyboardMarkup $reply_markup
   *
   * @return Array
   */
  public function sendVideo($chat_id, $video, $reply_to_message_id = null, $replyMarkup = null)
  {
    $data = compact('chat_id', 'video', 'reply_to_message_id', 'reply_markup');

    
    return $this->uploadFile('sendVideo', $data, $video);
  }

  /**
   * Send Location.
   *
   * @link https://core.telegram.org/bots/api#sendlocation
   *
   * @param int            $chat_id
   * @param float          $latitude
   * @param float          $longitude
   * @param int            $reply_to_message_id
   * @param KeyboardMarkup $reply_markup
   *
   * @return Array
   */
  public function sendLocation($chat_id, $latitude, $longitude, $reply_to_message_id = null, $reply_markup = null)
  {
    $params = compact('chat_id', 'latitude', 'longitude', 'reply_to_message_id', 'reply_markup');

    return $this->sendRequest('sendLocation', $params);
  }

  /**
   * Send Chat Action.
   *
   * @link https://core.telegram.org/bots/api#sendchataction
   *
   * @param int            $chat_id
   * @param string         $action
   *
   * @return Array
   */
  public function sendChatAction($chat_id, $action)
  {
    $actions = array(
    'typing',
    'upload_photo',
    'record_video',
    'upload_video',
    'record_audio',
    'upload_audio',
    'upload_document',
    'find_location',
    );
    if (isset($action) && in_array($action, $actions))
    {
      $params = compact('chat_id', 'action');
      return $this->sendRequest('sendChatAction', $params);
    }

    throw new TelegramException('Invalid Action! Accepted value: '.implode(', ', $actions));
  }

  /**
   * Get user profile photos.
   *
   * @link https://core.telegram.org/bots/api#getsserprofilephotos
   *
   * @param int            $chat_id
   * @param int            $offset
   * @param int            $limit
   *
   * @return Array
   */
  public function getUserProfilePhotos($user_id, $offset = null, $limit = null)
  {
        $param = compact('user_id', 'offset', 'limit');

        return $this->sendRequest('getUserProfilePhotos', $params);
  }

 /**
  * Set a Webhook to receive incoming updates via an outgoing webhook.
  *
  * @param string $url HTTPS url to send updates to.
  *
  * @return Array
  *
  */
 public function setWebhook($url)
 {
   if (filter_var($url, FILTER_VALIDATE_URL) === false)
     throw new TelegramException('Invalid URL provided');

   if (parse_url($url, PHP_URL_SCHEME) !== 'https')
      throw new TelegramException('Invalid URL, it should be a HTTPS url.');

   return $this->sendRequest('setWebhook', $url);
}

 /**
  * Returns webhook updates sent by Telegram.
  * Works only if you set a webhook.
  *
  * @see setWebhook
  *
  * @return Array
  */
 public function getWebhookUpdates()
 {
   $body = json_decode(file_get_contents('php://input'), true);

   return $body;
 }

 /**
  * Removes the outgoing webhook.
  *
  * @return Array
  */
 public function removeWebhook()
 {
   $url = '';
   return $this->sendRequest('setWebhook', compact('url'));
 }

 /**
  * Builds a custom keyboard markup.
  *
  * @link https://core.telegram.org/bots/api#replykeyboardmarkup
  *
  * @param array $keyboard
  * @param bool  $resize_keyboard
  * @param bool  $one_time_keyboard
  * @param bool  $selective
  *
  * @return string
  */
 public function replyKeyboardMarkup($keyboard, $resize_keyboard = false, $one_time_keyboard = false, $selective = false)
 {
    return json_encode(compact('keyboard', 'resize_keyboard', 'one_time_keyboard', 'selective'));
 }

 /**
  * Hide the current custom keyboard and display the default letter-keyboard.
  *
  * @link https://core.telegram.org/bots/api#replykeyboardhide
  *
  * @param bool $selective
  *
  * @return string
  */
 public static function replyKeyboardHide($selective = false)
 {
    $hide_keyboard = true;
    return json_encode(compact('hide_keyboard', 'selective'));
 }

 /**
  * Display a reply interface to the user (act as if the user has selected the bots message and tapped 'Reply').
  *
  * @link https://core.telegram.org/bots/api#forcereply
  *
  * @param bool $selective
  *
  * @return string
  */
 public static function forceReply($selective = false)
 {
    $force_reply = true;
    return json_encode(compact('force_reply', 'selective'));
 }

 function file_get_contents_curl($url) {
	$ch = curl_init();
 
	curl_setopt($ch, CURLOPT_HEADER, 0);
	curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1); //Set curl to return the data instead of printing it to the browser.
	curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
		curl_setopt($ch, CURLOPT_SSLVERSION, 6); //6 is for TLSV1.2

	$data = curl_exec($ch);
	curl_close($ch);
 
	return $data;
}
 
  private function sendRequest($method, $params)
  {
    return json_decode($this->file_get_contents_curl($this->baseURL . $method . '?' . http_build_query($params)), true);
  }
  
  private function curl_custom_postfields($ch, array $assoc = array(), array $files = array(), array $file_mimes = array()) {
    
    // invalid characters for "name" and "filename"
    static $disallow = array("\0", "\"", "\r", "\n");
    
    // build normal parameters
    foreach ($assoc as $k => $v) {
        $k = str_replace($disallow, "_", $k);
        $body[] = implode("\r\n", array(
            "Content-Disposition: form-data; name=\"{$k}\"",
            "",
            filter_var($v), 
        ));
    }
    
    // build file parameters
    foreach ($files as $k => $v) {
        switch (true) {
            case false === $v = realpath(filter_var($v)):
            case !is_file($v):
            case !is_readable($v):
                continue 2; // or return false, throw new InvalidArgumentException
        }
        $data = file_get_contents($v);
	    $exploded = explode(DIRECTORY_SEPARATOR, $v);
        $v = end($exploded);
        $k = str_replace($disallow, "_", $k);
        $v = str_replace($disallow, "_", $v);
        
        $mime = 'application/octet-stream';
        if(isset($file_mimes[$k])){
            $mime = $file_mimes[$k];
        }
        
        $body[] = implode("\r\n", array(
            "Content-Disposition: form-data; name=\"{$k}\"; filename=\"{$v}\"",
            "Content-Type: ".$mime,
            "",
            $data, 
        ));
    }
    
    // generate safe boundary 
    do {
        $boundary = "---------------------" . md5(mt_rand() . microtime());
    } while (preg_grep("/{$boundary}/", $body));
    
    // add boundary for each parameters
    array_walk($body, function (&$part) use ($boundary) {
        $part = "--{$boundary}\r\n{$part}";
    });
    
    // add final boundary
    $body[] = "--{$boundary}--";
    $body[] = "";
    
    // set options
    return @curl_setopt_array($ch, array(
        CURLOPT_POST       => true,
        CURLOPT_POSTFIELDS => implode("\r\n", $body),
        CURLOPT_HTTPHEADER => array(
            "Expect: 100-continue",
            "Content-Type: multipart/form-data; boundary={$boundary}", // change Content-Type
        ),
    ));
}
    
  private function uploadFile($method, $data, $photo = '')
  {
    $key = array(
      'sendPhoto'    => 'photo',
      'sendAudio'    => 'audio',
      'sendDocument' => 'document',
      'sendSticker'  => 'sticker',
      'sendVideo'    => 'video'
    );

    $file = $photo;
    
    $mime_type = $this->mime_content_type($file);
    
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, $this->baseURL . $method);
    curl_setopt($ch, CURLOPT_POST, 1);
	curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);
    $this->curl_custom_postfields($ch, $data, array($key[$method] => $file), array($key[$method] => $mime_type));
    curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
	curl_setopt($ch, CURLOPT_SSLVERSION, 6); //6 is for TLSV1.2


    $response = json_decode(curl_exec($ch), true);

    return $response;
  }
  
  function mime_content_type($filename) {

        $mime_types = array(

            'txt' => 'text/plain',
            'htm' => 'text/html',
            'html' => 'text/html',
            'php' => 'text/html',
            'css' => 'text/css',
            'js' => 'application/javascript',
            'json' => 'application/json',
            'xml' => 'application/xml',
            'swf' => 'application/x-shockwave-flash',
            'flv' => 'video/x-flv',

            // images
            'png' => 'image/png',
            'jpe' => 'image/jpeg',
            'jpeg' => 'image/jpeg',
            'jpg' => 'image/jpeg',
            'gif' => 'image/gif',
            'bmp' => 'image/bmp',
            'ico' => 'image/vnd.microsoft.icon',
            'tiff' => 'image/tiff',
            'tif' => 'image/tiff',
            'svg' => 'image/svg+xml',
            'svgz' => 'image/svg+xml',

            // archives
            'zip' => 'application/zip',
            'rar' => 'application/x-rar-compressed',
            'exe' => 'application/x-msdownload',
            'msi' => 'application/x-msdownload',
            'cab' => 'application/vnd.ms-cab-compressed',

            // audio/video
            'mp3' => 'audio/mpeg',
            'qt' => 'video/quicktime',
            'mov' => 'video/quicktime',

            // adobe
            'pdf' => 'application/pdf',
            'psd' => 'image/vnd.adobe.photoshop',
            'ai' => 'application/postscript',
            'eps' => 'application/postscript',
            'ps' => 'application/postscript',

            // ms office
            'doc' => 'application/msword',
            'rtf' => 'application/rtf',
            'xls' => 'application/vnd.ms-excel',
            'ppt' => 'application/vnd.ms-powerpoint',

            // open office
            'odt' => 'application/vnd.oasis.opendocument.text',
            'ods' => 'application/vnd.oasis.opendocument.spreadsheet',
        );

        $ex = explode('.',$filename);
        $popped = array_pop($ex);
        $ext = strtolower($popped);
        
        if (function_exists('finfo_open')) {
            $finfo = finfo_open(FILEINFO_MIME);
            $mimetype = finfo_file($finfo, $filename);
            finfo_close($finfo);
            return $mimetype;
        } else if (array_key_exists($ext, $mime_types)) {
            return $mime_types[$ext];
        }
        else {
            return 'application/octet-stream';
        }
    }

}